@@ -1,28 +1,22 @@
 #   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #
 #   file: lib/Dist/Zilla/PluginBundle/Author/VDB.pm
-
-#pod =encoding UTF-8
-#pod
-#pod =head1 COPYRIGHT AND LICENSE
-#pod
-#pod Copyright © 2015 Van de Bugger
-#pod
-#pod This file is part of perl-Dist-Zilla-PluginBundle-Author-VDB.
-#pod
-#pod perl-Dist-Zilla-PluginBundle-Author-VDB is free software: you can redistribute it and/or modify
-#pod it under the terms of the GNU General Public License as published by the Free Software
-#pod Foundation, either version 3 of the License, or (at your option) any later version.
-#pod
-#pod perl-Dist-Zilla-PluginBundle-Author-VDB is distributed in the hope that it will be useful, but
-#pod WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-#pod PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#pod
-#pod You should have received a copy of the GNU General Public License along with
-#pod perl-Dist-Zilla-PluginBundle-Author-VDB. If not, see <http://www.gnu.org/licenses/>.
-#pod
-#pod =cut
-
+#
+#   Copyright © 2015 Van de Bugger
+#
+#   This file is part of perl-Dist-Zilla-PluginBundle-Author-VDB.
+#
+#   perl-Dist-Zilla-PluginBundle-Author-VDB is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by the Free Software
+#   Foundation, either version 3 of the License, or (at your option) any later version.
+#
+#   perl-Dist-Zilla-PluginBundle-Author-VDB is distributed in the hope that it will be useful, but
+#   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+#   PARTICULAR PURPOSE. See the GNU General Public License for more details.
+#
+#   You should have received a copy of the GNU General Public License along with
+#   perl-Dist-Zilla-PluginBundle-Author-VDB. If not, see <http://www.gnu.org/licenses/>.
+#
 #   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 #pod =head1 DESCRIPTION
@@ -42,7 +36,7 @@ use version 0.77;
 
 # PODNAME: Dist::Zilla::PluginBundle::Author::VDB
 # ABSTRACT: VDB's plugin bundle
-our $VERSION = 'v0.10.2'; # VERSION
+our $VERSION = 'v0.11.0'; # VERSION
 
 with 'Dist::Zilla::Role::PluginBundle::Easy';
 
@@ -60,11 +54,6 @@ use File::chdir         ();
 use IPC::Run3           ();
 use IPC::System::Simple ();
 use Path::Tiny 0.070    ();
-use Perl::Version       ();
-
-#   `AutoPrereqs` hints: Implicitly required modules.
-use Pod::Weaver::PluginBundle::Author::VDB ();
-use Perl::PrereqScanner::Scanner::Hint     ();
 
 # --------------------------------------------------------------------------------------------------
 
@@ -110,7 +99,7 @@ has copying => (
 #pod =option readme
 #pod
 #pod Names of POD files to generate distribution F<README> text file. This is a multi-value option.
-#pod Empty values are ignored. Empty list disables generating F<README>file.
+#pod Empty values are ignored. Empty list disables generating F<README> file.
 #pod
 #pod C<ArrayRef[Str]>, optional, default value C<[ 'doc/what.pod', 'doc/why.pod', 'doc/naming.pod', 'doc/forms.pod', 'doc/source.pod', 'doc/distribution.pod', 'doc/installing.pod', 'doc/hacking.pod', 'doc/documentation.pod', 'doc/feedback.pod', 'doc/glossary.pod' ]>.
 #pod
@@ -207,7 +196,7 @@ has templates => (
 #pod
 #pod =option unwanted_modules
 #pod
-#pod TODO C<[ qw{ DPP Data::Printer } ]>.
+#pod TODO C<[ qw{ DDP Data::Printer } ]>.
 #pod
 #pod =cut
 
@@ -220,7 +209,7 @@ has unwanted_modules => (
         my $p  = $self->payload;
         my $u1 = $p->{ unwanted_module  };
         my $u2 = $p->{ unwanted_modules };
-        return $u1 || $u2 ? [ $u1 ? @$u1 : (), $u2 ? @$u2 : () ] : [ qw{ DPP Data::Printer } ];
+        return $u1 || $u2 ? [ $u1 ? @$u1 : (), $u2 ? @$u2 : () ] : [ qw{ DDP Data::Printer } ];
     },
 );
 
@@ -250,12 +239,49 @@ has spellchecker => (
 
 # --------------------------------------------------------------------------------------------------
 
+#pod =option repository
+#pod
+#pod Mercurial repository to push changes after release to. Option may be specified multiple times to
+#pod push changes into several repositories. By default changes are pushed to one repository C<default>.
+#pod
+#pod =cut
+
+has repository => (
+    isa         => 'ArrayRef[Str]',
+    is          => 'ro',
+    lazy        => 1,
+    default     => sub {
+        my ( $self ) = @_;
+        return $self->payload->{ repository } // [ 'default' ];
+    },
+);
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =option installer
+#pod
+#pod TODO
+#pod
+#pod =cut
+
+has installer => (
+    isa         => 'Str',
+    is          => 'ro',
+    lazy        => 1,
+    default     => sub {
+        my ( $self ) = @_;
+        return $self->payload->{ installer } // 'ModuleBuildTiny';
+    },
+);
+
+# --------------------------------------------------------------------------------------------------
+
 #pod =method mvp_multivalue_args
 #pod
 #pod =cut
 
 sub mvp_multivalue_args {
-    return qw{ templates unwanted_module unwanted_modules readme };
+    return qw{ templates unwanted_module unwanted_modules readme repository };
 };
 
 # --------------------------------------------------------------------------------------------------
@@ -325,6 +351,7 @@ sub configure {
 
     my ( $self ) = @_;
     our $Self = $self;                  ## no critic ( ProhibitPackageVars )
+    my $name = $self->name;
 
     require Dist::Zilla::Plugin::Manifest::Write;
         # Make sure the plugin is loaded before checking its version.
@@ -332,7 +359,7 @@ sub configure {
     $self->add_plugins(
 
         [ 'Hook' => 'prologue' => {
-            # DOES NOT WORK because plugin name will be '@Author::VDB/prologue'.
+            # DOES NOT WORK because plugin name will be '$name/prologue'.
             'hook' => [ q{
                 use autodie ':all';
                 use IPC::System::Simple qw{ capture };
@@ -340,19 +367,7 @@ sub configure {
             } ],
         } ],
 
-        [ 'Hook::VersionProvider' => {
-            'hook' => [ q{
-                use Path::Tiny;
-                my $file = path( $zilla->root )->child( 'VERSION' );
-                ( my $version = $file->slurp ) =~ s{\s+\z}{};
-                if ( $version !~ m{\A@{ [ Perl::Version::REGEX ] }\z} ) {
-                    $self->abort( [
-                        'Invalid version string "%s" at %s line 1', $version, "$file"
-                    ] );
-                };
-                $version;
-            } ],
-        } ],
+        [ 'Author::VDB::Version::Read', ],
 
         [ 'Hook::Init' => 'init stuff' => {
             'hook' => [ q[
@@ -380,16 +395,6 @@ sub configure {
             ] ],
         } ],
 
-        #   I would like to set `is_strict` option to `1`, but it will fail for trial releases.
-        #   To avoid that let's do a trick: set `is_strict` to `1` only in case of non-trial
-        #   release.
-        [ 'Hook::BeforeBuild' => 'hack Test::Version' => {
-            'hook' => [ q[
-                my $tv = $zilla->plugin_named( '@Author::VDB/Test::Version' );
-                $tv->{ is_strict } = $dist->is_trial ? '0' : '1';
-            ] ],
-        } ],
-
         #
         #   Files to include
         #
@@ -431,7 +436,7 @@ sub configure {
                     q[                ->pod2text( width => 80, indent => 0, loose => 1, quotes => 'none' ) ],
                     q[                ->chomp                                                              ],
                     q[        }                                                                            ],
-                    q[    ] . $self->_quote( $self->readme ) . q[                                          ],
+                    q[        ] . $self->_quote( $self->readme ) . q[                                      ],
                     q[    )                                                                                ],
                     q[);                                                                                   ],
                     q[}] . q[}],        # One newline will be added there.
@@ -442,26 +447,17 @@ sub configure {
 
         [ 'Manifest::Write' => { # REQUIRED VERSION: v0.9.0
             # `Manifest::Write` v0.9.0 strictly requires plugin names, not monikers.
-            # If a plugin name is added from the bundle, `PluginBundle::Easy` prepends its name
-            # with the bundle name (or moniker?).
-            Dist::Zilla::Plugin::Manifest::Write->VERSION() < version->parse( 'v0.9.0' ) ? (
-                'source_providers' => [
-                    'Manifest::Read',
-                    'GenerateFile',
-                ],
-            ) : (
-                'source_provider' => [
-                    '@Author::VDB/Manifest::Read',
-                    $self->copying ne '' ? '@Author::VDB/COPYING' : (),
-                    $self->readme        ? '@Author::VDB/README'  : (),
-                ],
-                'metainfo_provider' => [
-                    # Defaults are not suitable because they are just `MetaJSON` and `MetaYAML`.
-                    '@Author::VDB/Manifest::Write',
-                    '@Author::VDB/MetaJSON',
-                    '@Author::VDB/MetaYAML',
-                ],
-            ),
+            'source_provider' => [
+                "$name/Manifest::Read",
+                $self->copying ne '' ? "$name/COPYING" : (),
+                $self->readme        ? "$name/README"  : (),
+            ],
+            'metainfo_provider' => [
+                # Defaults are not suitable because they are just `MetaJSON` and `MetaYAML`.
+                "$name/Manifest::Write",
+                "$name/MetaJSON",
+                "$name/MetaYAML",
+            ],
         } ],
 
         #
@@ -470,7 +466,7 @@ sub configure {
 
         [ 'Templates' => { # REQUIRED VERSION: v0.5.0 # for including `Dist::Zilla::File` objects.
             'templates' => [
-                '@Author::VDB/src doc',
+                "$name/src doc",
                 @{ $self->templates // [] },
             ],
         } ],
@@ -478,12 +474,12 @@ sub configure {
         [ 'OurPkgVersion' ],
 
         [ 'SurgicalPodWeaver' => {
-            'config_plugin' => '@Author::VDB',
+            'config_plugin' => '@Author::VDB',  # REQUIRE: Pod::Weaver::PluginBundle::Author::VDB
             'replacer'      => 'replace_with_comment',
         } ],
 
         [ 'FileFinder::ByName' => 'src doc' => {
-            # Plugin name will be `@Author::VDB/doc`.
+            # Plugin name will be `$name/doc`.
             'file' => [
                 $self->copying ne '' ? ( 'COPYING' ) : (),
                 $self->readme        ? ( 'README'  ) : (),
@@ -498,7 +494,7 @@ sub configure {
         [ 'Hook::AfterBuild' => 'update src doc' => {
             'hook' => [ q{
                 use Path::Tiny;
-                my $files = $zilla->plugin_named( '@Author::VDB/src doc' )->find_files();
+                my $files = $zilla->plugin_named( '} . $name . q{/src doc' )->find_files();
                 my $build = path( $arg->{ build_root } );
                 my $root  = path( $dist->root );
                 for my $file ( @$files ) {
@@ -508,7 +504,8 @@ sub configure {
                     my $old_bulk = $old_file->exists ? $old_file->slurp : undef;
                     if ( not defined( $old_bulk ) or $new_bulk ne $old_bulk ) {
                         $self->log( [ 'updating %s', $file->name ] );
-                        $old_file->spew( $new_bulk );
+                        $old_file->append( { truncate => 1 }, $new_bulk );
+                            # `append` is not atomic, but does not reset file mode.
                     };
                 };
             } ],
@@ -525,11 +522,11 @@ sub configure {
         [ 'Test::Portability' ],        # Checks filenames.
 
         [ 'Test::EOL' => {
-            'finder' => '@Author::VDB/Manifest::Read',
+            'finder' => "$name/Manifest::Read",
         } ],
 
         [ 'Test::NoTabs' => {
-            'finder' => '@Author::VDB/Manifest::Read',
+            'finder' => "$name/Manifest::Read",
         } ],
 
         [ 'MojibakeTests' ],
@@ -541,7 +538,19 @@ sub configure {
         } ],
 
         [ 'Test::Version' => {          # All modules have version.
-            'is_strict' => 0,           # Strict version test fails in trial releases.
+            finder    => "$name/Manifest::Read/InstallModules",
+                # REQUIRE: Dist::Zilla::Plugin::Manifest::Read v0.4.0 # want `/InstallModules`.
+            is_strict => 0,           # Strict version test fails in trial releases.
+        } ],
+
+        #   I would like to set `Test::Version`'s `is_strict` option to `1`, but it will fail for
+        #   trial releases. To avoid that let's do a trick: set `is_strict` to `1` only in case of
+        #   non-trial release.
+        [ 'Hook::BeforeBuild' => 'hack Test::Version' => {
+            'hook' => [ q{
+                my $tv = $zilla->plugin_named( '} . $name . q{/Test::Version' );
+                $tv->{ is_strict } = $dist->is_trial ? '0' : '1';
+            } ],
         } ],
 
         [ 'Test::NewVersion' ],         # This is not a version already uploaded to CPAN.
@@ -599,7 +608,7 @@ sub configure {
         [ 'MinimumPerl' ],
 
         [ 'AutoPrereqs' => {
-            'extra_scanners' => 'Hint',
+            'extra_scanners' => 'Hint', # REQUIRE: Perl::PrereqScanner::Scanner::Hint
         } ],
 
         #   `Prereqs::AuthorDeps` has a problem:
@@ -678,7 +687,10 @@ sub configure {
         #   Installer
         #
 
-        [ 'ModuleBuildTiny' ],
+        $self->installer ? (
+            [ $self->installer ],
+        ) : (
+        ),
 
         #
         #   Release
@@ -756,7 +768,7 @@ sub configure {
                     #
                     #   Steal list of words to ignore from `PodSpelling` plugin.
                     #
-                    my $podspelling = $zilla->plugin_named( '@Author::VDB/Test::PodSpelling' );
+                    my $podspelling = $zilla->plugin_named( '} . $name . q{/Test::PodSpelling' );
                     my %ignore = map( { $_ => 1 } @{ $podspelling->stopwords } );
                     #
                     #   Add all module names.
@@ -791,38 +803,9 @@ sub configure {
         ) : (
         ),
 
-        #   Make sure there is no such tag yet, neither global nor local.
-        #   Is seems direct scanning `.hgtags` and hg/localtags` does not always work in case of
-        #   branches. Parsing `hg tag` may be more correct.
-        #   TODO: Check it.
-        [ 'Hook::BeforeRelease' => 'hg check tag' => {
-            'hook' => [ q{
-                use Path::Tiny;
-                my $version = $dist->version;
-                my $tags = path( '.hgtags' );
-                if ( $tags->slurp_utf8 =~ m{ \Q$version\E$}m ) {
-                    $self->abort( [ 'tag %s already exists', $version ] );
-                };
-                my $ltags = path( '.hg/localtags' );
-                if ( $ltags->exists and $ltags->slurp_utf8 =~ m{ \Q$version\E$}m ) {
-                    $self->abort( [ 'local tag %s already exists', $version ] );
-                };
-            } ],
-        } ],
+        [ 'Author::VDB::Hg::Tag::Check', ],
 
-        [ 'Hook::BeforeRelease' => 'hg stat' => {
-            'hook' => [ q{
-                #   Make sure all the files committed.
-                use IPC::System::Simple 'capture';
-                my @stdout = capture( 'hg', 'status' );
-                if ( @stdout ) {
-                    chomp( @stdout );
-                    $self->log_error( 'hg status:' );
-                    $self->log_error( "    $_" ) for @stdout;
-                    $self->abort( 'hg status is not clean' );
-                };
-            } ],
-        } ],
+        [ 'Author::VDB::Hg::Status', ],
 
         $self->local_release ? (
             [ 'Hook::BeforeRelease' => 'release note' => {
@@ -861,59 +844,27 @@ sub configure {
             [ 'UploadToCPAN' ],
         ),
 
-        [ 'Hook::AfterRelease' => 'hg add tag' => {
-            'hook' => [ q{
-                use IPC::System::Simple 'capture';
-                my @stdout = capture( 'hg', 'id', '--debug' );
-                chomp( @stdout );
-                @stdout == 1 and $stdout[ 0 ] =~ m{^([0-9a-f]{40})(\+?) tip$} or do {
-                    $self->log_error( 'fail to parse "hg id" output:' );
-                    $self->log_error( "    $_" ) for @stdout;
-                    $self->abort();
-                };
-                my ( $id, $dirty ) = ( $1, $2 );
-                if ( $dirty ) {
-                    $self->abort( 'hg status is not clean' );
-                };
-                path( '} . ( $self->local_release ? '.hg/localtags' : '.hgtags' ) . q{' )->append( sprintf( "%s %s\n", $id, $dist->version ) );
-            } ],
-        } ],
+        [ 'Author::VDB::Hg::Tag::Add', ],
 
         [ 'NextRelease' => {
             'format'      => '%V @ %{yyyy-MM-dd HH:mm zzz}d',
             'time_zone'   => 'UTC',
         } ],
 
-        [ 'Hook::AfterRelease' => 'bump version' => {
-            'hook' => [ q{
-                use Path::Tiny;
-                use Perl::Version;
-                my $version = Perl::Version->new( $dist->version );
-                $version->inc_alpha();
-                path( $zilla->root )->child( 'VERSION' )->spew( $version );
-                $self->log( [ 'next release will be %s', "$version" ] );
-            } ],
-        } ],
+        [ 'Author::VDB::Version::Bump', ],
 
-        [ 'Hook::AfterRelease' => 'hg post-commit' => {
-            'hook' => [ q{
-                use autodie ':all';
-                system(
-                    'hg', 'commit', '-m', 'post-release',
-                        qw{ .hgtags Changes VERSION }
-                );
-            } ],
+        [ 'Author::VDB::Hg::Commit', {
+            'files' => [
+                '.hgtags',      # Changed by `Hg::Tag::Add`.
+                'Changes',      # Changed by `NextRelease`.
+                'VERSION',      # Changed by `Version::Bump`.
+            ],
         } ],
 
         $self->local_release ? (
         ) : (
-            [ 'Hook::AfterRelease' => 'hg push' => {
-                'hook' => [ q{
-                    use IPC::System::Simple 'capture';
-                    my @stdout = capture( 'hg', 'push' );
-                    chomp( @stdout );
-                    $self->log( "    $_" ) for @stdout;
-                } ],
+            [ 'Author::VDB::Hg::Push', {
+                repository => $self->repository,
             } ],
         ),
 
@@ -945,6 +896,19 @@ __PACKAGE__->meta->make_immutable();
 
 1;
 
+#pod =head1 COPYRIGHT AND LICENSE
+#pod
+#pod Copyright (C) 2015 Van de Bugger
+#pod
+#pod License GPLv3+: The GNU General Public License version 3 or later
+#pod <http://www.gnu.org/licenses/gpl-3.0.txt>.
+#pod
+#pod This is free software: you are free to change and redistribute it. There is
+#pod NO WARRANTY, to the extent permitted by law.
+#pod
+#pod
+#pod =cut
+
 # doc/what.pod #
 
 #pod =encoding UTF-8
@@ -980,7 +944,7 @@ Dist::Zilla::PluginBundle::Author::VDB - VDB's plugin bundle
 
 =head1 VERSION
 
-Version v0.10.2, released on 2015-10-22 21:30 UTC.
+Version v0.11.0, released on 2015-11-05 21:42 UTC.
 
 =head1 WHAT?
 
@@ -1016,7 +980,7 @@ C<Str>, optional, default value C<doc/copying.pod>.
 =head2 readme
 
 Names of POD files to generate distribution F<README> text file. This is a multi-value option.
-Empty values are ignored. Empty list disables generating F<README>file.
+Empty values are ignored. Empty list disables generating F<README> file.
 
 C<ArrayRef[Str]>, optional, default value C<[ 'doc/what.pod', 'doc/why.pod', 'doc/naming.pod', 'doc/forms.pod', 'doc/source.pod', 'doc/distribution.pod', 'doc/installing.pod', 'doc/hacking.pod', 'doc/documentation.pod', 'doc/feedback.pod', 'doc/glossary.pod' ]>.
 
@@ -1047,7 +1011,7 @@ Optional, default value C<[ ':InstallModules' ]>.
 
 =head2 unwanted_modules
 
-TODO C<[ qw{ DPP Data::Printer } ]>.
+TODO C<[ qw{ DDP Data::Printer } ]>.
 
 =head2 spellchecker
 
@@ -1057,6 +1021,15 @@ plugin and internally implemented checking the F<Changes> file.
 
 Optional, default value C<aspell list -l en -p ./xt/aspell-en.pws>.
 
+=head2 repository
+
+Mercurial repository to push changes after release to. Option may be specified multiple times to
+push changes into several repositories. By default changes are pushed to one repository C<default>.
+
+=head2 installer
+
+TODO
+
 =head1 WHY?
 
 I have published few distributions on CPAN. Every distribution have F<dist.ini> file. All the
@@ -1071,20 +1044,13 @@ Van de Bugger <van.de.bugger@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2015 Van de Bugger
+Copyright (C) 2015 Van de Bugger
 
-This file is part of perl-Dist-Zilla-PluginBundle-Author-VDB.
+License GPLv3+: The GNU General Public License version 3 or later
+<http://www.gnu.org/licenses/gpl-3.0.txt>.
 
-perl-Dist-Zilla-PluginBundle-Author-VDB is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by the Free Software
-Foundation, either version 3 of the License, or (at your option) any later version.
-
-perl-Dist-Zilla-PluginBundle-Author-VDB is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License along with
-perl-Dist-Zilla-PluginBundle-Author-VDB. If not, see <http://www.gnu.org/licenses/>.
+This is free software: you are free to change and redistribute it. There is
+NO WARRANTY, to the extent permitted by law.
 
 =cut
 
